   
      module mpi_data_module

************************************************************************
*  
*     Name: mpi_data_module.
*
*     Usage:
*
*     use mpi_data_module
*
*     Revision History:
*
*     20-Dec-03 -- SAF; initial construction of mpi_data_module.
*     22-Dec-03 -- SAF; incorporated into mpi_data.inc
*     23-Dec-03 -- SAF; add announce() to module. 
*       
************************************************************************

      implicit none
      save 

      include "mpif.h"

      integer (kind = 4), parameter :: MPI_FLPT = MPI_FLPT_PRECISION
      integer (kind = 4), parameter :: MASTER_RANK = 0
      integer (kind = 4) :: my_rank, num_procs, mpi_ierr

      CONTAINS

         subroutine MPI_Begin 
************************************************************************
*  
*        Name: MPI_Begin
*
*        Purpose: Initialize the MPI library.
*
*        Revision History:
*
*        10-Mar-02 -- SAF; initial development of MPI_Begin().
*        22-Dec-03 -- SAF; incorporated into mpi_data_module.
*
************************************************************************

         implicit none

*        Begin MPI_Begin.

         call MPI_Init (mpi_ierr)
         call MPI_Comm_rank (MPI_COMM_WORLD, my_rank,   mpi_ierr)
         call MPI_Comm_size (MPI_COMM_WORLD, num_procs, mpi_ierr)

*        End MPI_Begin.

         return
         end subroutine MPI_Begin

         subroutine MPI_End 
************************************************************************
*  
*        Name: MPI_End
*
*        Purpose: Terminate the MPI library.
*
*        Revision History:
*
*        10-Mar-02 -- SAF; initial development of MPI_End().
*        22-Dec-03 -- SAF; incorporated into mpi_data_module.
*
************************************************************************

         implicit none

*        Begin MPI_End.

         call MPI_Finalize (mpi_ierr)

*        End MPI_End.

         return
         end subroutine MPI_End

         subroutine announce 
************************************************************************
*
*        Name: announce
*
*        Purpose: print rank, hostname and pid to <stdout>. Called mostly
*                 in debugging situations.
*
*        Revision History:
*
*        13-Jun-02 -- SAF; add header documentation.
*        20-Dec-03 -- SAF; add MPI_Barrier call to ensure that all the
*                          output lines make it out immediately.
*        22-Dec-03 -- SAF; use mpi_data_module and flush_unit_module.
*        23-Dec-03 -- SAF; added to the mpi_data_module.inc.
*
************************************************************************

         use flush_unit_module

         implicit none

         integer (kind = 4) :: reslen, getpid
         character (len = MPI_MAX_PROCESSOR_NAME) :: hostname

*        Begin announce.

*        Get the name of this processor.

         call MPI_Get_processor_name (hostname, reslen, mpi_ierr)

*        Write out the announce line and flush the buffer.

         write (6, 600) my_rank, hostname(1:reslen), getpid()
         call flush_unit (6)

*        Hold everyone up so that the announce lines make it out.

         call MPI_Barrier (MPI_COMM_WORLD, mpi_ierr)

*        End announce.

 600     format ("announce: rank = ", i2, ", host = ", a,
     &           ", pid = ", i12)

         return
         end subroutine announce

      end module mpi_data_module
